@@ -24,6 +24,10 @@
 #'   since the method stops if the result does not change between scans.
 #' @param nPermutations The desired number of permutations to consider when
 #'   searching for the minimizer.
+#' @param probExploration The expected probability of picking the second best
+#'   micro-optimization (instead of the best).  For a given permutation, the
+#'   probability is sampled from a beta distribution with shape \code{1} and
+#'   \code{1/probExploration}.
 #' @param seconds A time threshold in seconds after which the function will
 #'   return early (with a warning) instead of finishing all the desired
 #'   permutations. Note that the function could take considerably longer,
@@ -71,7 +75,7 @@
 #' probs <- psm(iris.clusterings, parallel=FALSE)
 #' salso(probs, nPermutations=50, parallel=FALSE)
 #'
-salso <- function(psm, loss=c("VI.lb","binder")[1], maxSize=0, maxScans=5, nPermutations=5000, seconds=10, parallel=TRUE) {
+salso <- function(psm, loss=c("VI.lb","binder")[1], maxSize=0, maxScans=5, nPermutations=5000, probExploration=0.005, seconds=10, parallel=TRUE) {
   if ( ! ( isSymmetric(psm) && all(0 <= psm) && all(psm <= 1) && all(diag(psm)==1) ) ) {
     stop("'psm' should be symmetric with diagonal elements equal to 1 and off-diagonal elements in [0, 1].")
   }
@@ -80,7 +84,8 @@ salso <- function(psm, loss=c("VI.lb","binder")[1], maxSize=0, maxScans=5, nPerm
   if ( maxScans < 0 ) stop("'maxScans' may not be negative.")
   if ( nPermutations < 0 ) stop("'nPermutations' may not be negative.")
   useVIlb <- loss == "VI.lb"
-  y <- .Call(.minimize_by_salso, nrow(psm), psm, useVIlb, maxSize, maxScans, nPermutations, seconds, parallel)
+  seed <- sapply(1:32, function(i) sample.int(256L,1L)-1L)
+  y <- .Call(.minimize_by_salso, nrow(psm), psm, useVIlb, maxSize, maxScans, nPermutations, probExploration, seconds, parallel, seed)
   names(y) <- c("estimate","loss","expectedLoss","nScans","nPermutations")
   names(y[[1]]) <- colnames(psm)
   y[[2]] <- loss
